@@ -11,22 +11,37 @@ ivector_minus_one <- function( ivector ){
 
 # Modified from sdmTMB::make_anisotropy_spde, originally from geostatistical_delta-GLMM
 make_anisotropy_spde <-
-function( inla_mesh ){
+function( inla_mesh,
+          covariates ){
 
   spde = fm_fem( inla_mesh )
-  Dset <- 1:2
+  if( missing(covariates) ){
+    loc = inla_mesh$loc[,1:2]
+  }else{
+    if(nrow(inla_mesh$loc) != nrow(covariates)) stop("Check `covariates` in `make_anisotropy_spde`")
+    loc = cbind( inla_mesh$loc[,1:2], covariates )
+  }
+
+  # Extract vertices
   TV <- inla_mesh$graph$tv
-  V0 <- inla_mesh$loc[TV[, 1], Dset]
-  V1 <- inla_mesh$loc[TV[, 2], Dset]
-  V2 <- inla_mesh$loc[TV[, 3], Dset]
-  E0 <- V2 - V1
-  E1 <- V0 - V2
-  E2 <- V1 - V0
+  V0 <- loc[ TV[,1], ]
+  V1 <- loc[ TV[,2], ]
+  V2 <- loc[ TV[,3], ]
+
+  # Fill NAs as needed
+  replace_NA = function(m) ifelse(is.na(m),0,m)
+  E0 <- replace_NA(V2 - V1)
+  E1 <- replace_NA(V0 - V2)
+  E2 <- replace_NA(V1 - V0)
+
+  # Get triangle areas
   TmpFn <- function(Vec1, Vec2) abs(det(rbind(Vec1, Vec2)))
   Tri_Area <- rep(NA, nrow(E0))
   for (i in seq_len(length(Tri_Area))){
-    Tri_Area[i] <- TmpFn(E0[i,], E1[i,])/2
+    Tri_Area[i] <- TmpFn( E0[i,1:2], E1[i,1:2] ) / 2
   }
+
+  # Return it all
   ret <- list( n_s = inla_mesh$n,
                n_tri = nrow(TV),
                Tri_Area = Tri_Area,
@@ -44,16 +59,17 @@ function( inla_mesh ){
 # if H = diag(2), G is expected to match `fmesher::fm_fem(mesh)$g1`
 make_stiffness <-
 function( mesh,
-          loc = mesh$loc[,1:2], # Can swap in different values
-          H = diag(ncol(loc)) ){
+          loc, # Can swap in different values
+          H ){
 
   # local objects to simplify code
+  if(missing(loc)) loc = mesh$loc[,1:2]
+  if(missing(H)) H = diag( rep(1,ncol(loc)) )
   tv = mesh$graph$tv
   n = mesh$n
   adjH = solve(H) * det(H)
 
   # Extract edge vectors
-  if(missing(loc)) loc = mesh$loc
   v0 = loc[ tv[,1], ]
   v1 = loc[ tv[,2], ]
   v2 = loc[ tv[,3], ]
@@ -184,4 +200,3 @@ function( model ){
     list(endogenous = names(variables[variables]), exogenous = names(variables[!variables]))
 }
 
-
